#include "image.h"
#include <math.h>

Image::Image(){
    bytes = 0;
}


Image::Image(int taille_x, int taille_y, BYTE r, BYTE v, BYTE b)
{
    width = taille_x;
    height = taille_y;
    taille_ligne_memoire = ((int)ceil( (width*3.0)/4.0 ))*4;
    bytes = 0;
    allocMem();
    // on efface l'image
    for (int i=0 ; i<taille_ligne_memoire*height ; i++)
    {
        bytes[i] = 0;
    }
    if ((r!=0) || (v!=0) || (b!=0)) // on colorise l'image
    {
        for (int y=0 ; y<height ; y++)
        {
            for (int x=0 ; x<width ; x++)
            {
                set_pixel(x, y, 0, b);
                set_pixel(x, y, 1, v);
                set_pixel(x, y, 2, r);
            }
        }
    }
}


Image::~Image()
{
    delete[] bytes;
}


BYTE& Image::pixel(int x,int y,int c)
{
    return bytes[((height-y-1)*taille_ligne_memoire+(3*x))+c]; // Pour avoir une image non inverse verticalement
}


void Image::set_pixel(int x,int y, int c, BYTE couleur)
{
    bytes[((height-y-1)*taille_ligne_memoire+(3*x))+c] = couleur;    // Pour avoir une image non inverse verticalement
}


void Image::allocMem(){
    delete[] bytes;
    bytes=new BYTE[taille_ligne_memoire*height];
}


std::string Image::LibelleAnomalie(Image_Anomalie err)
{
    switch(err)
    {
    case(Image_ANO_NONBMP):
        return "Le fichier n'est pas une image au format BMP.";
    case(Image_ANO_OPENFIC):
        return "Impossible d'ouvrir le fichier.";
    case(Image_ANO_LECTUREFIC):
        return "Erreur de lecture.";
    case(Image_ANO_TAILLEINT):
        return "Taille d'entier incompatible avec cette machine.";
    case(Image_OK):
        return "OK";
    case(Image_ANO_FORMAT):
        return "Format d'image non gr.";
    default:
        return "Anomalie inconnue.";
    }
}


Image_Anomalie Image::ChargeBMP(char fname[])
{
    if(sizeof(int)!=4) return Image_ANO_TAILLEINT;


 	FILE* f=fopen(fname,"rb");      //open for reading in binary mode
    if(!f) return Image_ANO_OPENFIC;
    char header[54];
    fread(header,54,1,f);           //read the 54bit main header

    if(header[0]!='B' || header[1]!='M')
    {
        fclose(f);
        return Image_ANO_NONBMP;       //all bitmaps should start "BM"
    }

    int offset=*(unsigned int*)(header+10);

    width=*(int*)(header+18);
    height=*(int*)(header+22);
    //now the bitmap knows how big it is it can allocate its memory
    taille_ligne_memoire = ((int)ceil( (width*3.0)/4.0 ))*4;
    allocMem();

    int bits=int(header[28]);       //colourdepth

    int x,y,c;
    switch(bits)
    {
    case(24):
        fseek(f,offset,SEEK_SET);
        fread(bytes,taille_ligne_memoire*height,1,f);
        break;

    default:
        fclose(f);
        return Image_ANO_FORMAT;
    }
    if(ferror(f))
    {
        fclose(f);
        return Image_ANO_LECTUREFIC;
    }

   fclose(f);
    return Image_OK;
}


void Image::CopieImage(Image& image_src, int pos_copie_x, int pos_copie_y, int pos_source_x, int pos_source_y) //Julian
{
    if ((pos_copie_x+1 > width) || (pos_copie_y+1 > height)) return; // on est en dehors de l'image ; +1 car l'image commence  0

    for (int y_src=pos_source_x ; (y_src<image_src.height) && (y_src+pos_copie_y<height) ; y_src++)
    {
        for (int x_src=pos_source_y ; (x_src<image_src.width) && (x_src+pos_copie_x<width) ; x_src++)
        {
            if (x_src+pos_copie_x>=0 && y_src+pos_copie_y>=0)
            {
                set_pixel(x_src+pos_copie_x, y_src+pos_copie_y, 0, image_src.pixel(x_src, y_src, 0));
                set_pixel(x_src+pos_copie_x, y_src+pos_copie_y, 1, image_src.pixel(x_src, y_src, 1));
                set_pixel(x_src+pos_copie_x, y_src+pos_copie_y, 2, image_src.pixel(x_src, y_src, 2));
            }
        }
    }
}


char Image::SauveBMP(char fname[])
{
    FILE* f=fopen(fname,"wb");
    if(!f) return Image_ANO_OPENFIC;

    char header[54];
    for (int i=0 ; i<54 ; i++) header[i]=0x00;

    header[0] = 0x42;
    header[1] = 0x4D;
    (int &)header[2] = (int)(taille_ligne_memoire*height + 54);
    (int &)header[10] = (int)54;
    (int &)header[14] = (int)40;
    (int &)header[18] = (int)width;
    (int &)header[22] = (int)height;
    header[26] = (char)01;
    header[28] = (char)24;
    (int &)header[34] = (int)(taille_ligne_memoire*height);

    fwrite(header, 1, 54, f);
    fwrite(bytes, 1, taille_ligne_memoire*height, f);

    fclose(f);
    return Image_OK;
}
