/*
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *

 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *

 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *

 * See at the end of this file for a copy of the GNU General Public License. *
 * Please observe modification and use guidelines of the GPL Licence.        *
 */

/*
 * Julian VASSAUX - 2011
 */

#include <iostream>
#include <fstream>
#include <dirent.h>
#include "image.h"
#include "list.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string>

using namespace std;


#define SCANtoEBOOK_VERSION "1.4"
#define SCANtoEBOOK_AUTHOR "Julian VASSAUX - http://www.vassaux.net"

#define FORMAT_NON_SPECIFIE 0
#define FORMAT_EPUB 1
#define FORMAT_LRF 2

#define MODELE_ECRAN_SONY		1
#define MODELE_ECRAN_SAMSUNG	2
#define LARGEUR_SONY	565
#define HAUTEUR_SONY	754
#define LARGEUR_SAMSUNG	565
#define HAUTEUR_SAMSUNG	668

#define CROPPING_TOP_LIGHT 2
#define CROPPING_TOP_MEDIUM 6
#define CROPPING_TOP_STRONG 10
#define CROPPING_BOTTOM_LIGHT 2
#define CROPPING_BOTTOM_MEDIUM 6
#define CROPPING_BOTTOM_STRONG 10


/*
 * Directory structure
 */
struct struct_arborescence
{
	int hierarchie;
	int numero_image;
	char * nom_lien;
};
struct_arborescence arborescence[99999]; //hardcoded 99999 max...
int nombre_images_par_page;

/*
 * Check if file exists
 */
bool FileExists(string strFilename) {
  struct stat stFileInfo;
  bool blnReturn;
  int intStat;

  // Attempt to get the file attributes
  intStat = stat(strFilename.c_str(),&stFileInfo);
  if(intStat == 0) {
    // File exists
    blnReturn = true;
  } else {
    // File does not exists
    blnReturn = false;
  }

  return(blnReturn);
}

/*
 * List directory content
 */
void lister_repertoire(List<char*> *liste_fichiers_jpg, const char* param_repertoire, int param_niveau)
{
	char chaine[10000];
	struct stat st;

    DIR *dp;
    struct dirent *ep;
    dp = opendir (param_repertoire);
    if (dp != NULL)
    {
        while (ep = readdir (dp))
        {

			sprintf(chaine, "%s\\%s", param_repertoire, ep->d_name);
			int statut = stat(chaine, &st);
			if(S_ISDIR(st.st_mode))
			{
				//This is a directory
				if ((strcmp(ep->d_name, ".")==0) || (strcmp(ep->d_name, "..")==0))
				{
					// we do not read these directories.
				}
				else
				{
					arborescence[liste_fichiers_jpg->getSize()*nombre_images_par_page].hierarchie=param_niveau;
					arborescence[liste_fichiers_jpg->getSize()*nombre_images_par_page].numero_image=liste_fichiers_jpg->getSize();
					char *nom_lien = new char[strlen(ep->d_name)+1];
					sprintf(nom_lien, "%s", ep->d_name);
					arborescence[liste_fichiers_jpg->getSize()*nombre_images_par_page].nom_lien=nom_lien;
					sprintf(chaine, "%s\\%s", param_repertoire, ep->d_name);
					lister_repertoire(liste_fichiers_jpg, chaine, param_niveau+1);
				}
			}
			else
			{
				// We have a file
				int longueur = strlen(ep->d_name);
				int longueur2 = strlen(param_repertoire);
				// We check this is a JPG file
				if ((ep->d_name[longueur-4] == '.') && ((ep->d_name[longueur-3] == 'j')||(ep->d_name[longueur-3] == 'J')) && ((ep->d_name[longueur-2] == 'p')||(ep->d_name[longueur-2] == 'P')) && ((ep->d_name[longueur-1] == 'g')||(ep->d_name[longueur-1] == 'G')))
				{
					// add image
					char *fichier_jpg = new char[longueur+1-4+1+strlen(param_repertoire)+1];
					sprintf(fichier_jpg, "%s\\", param_repertoire);
					for (int i=0 ; i<longueur+1-4-1 ; i++) fichier_jpg[i+longueur2+1] = ep->d_name[i]; //(sans extension)
					fichier_jpg[longueur+1-4-1+longueur2+1]=0;
					liste_fichiers_jpg->append(fichier_jpg);
				}
			}
		}
	}
}


int main()
{

	cout << "#####################################################################"<<endl;
	cout << "##                                                                 ##"<<endl;
	cout << "##  SCANtoEBOOK v" << SCANtoEBOOK_VERSION <<" 2011  Julian VASSAUX                         ##"<<endl;
	cout << "##  http://scantoebook.vassaux.net                                 ##"<<endl;
	cout << "##                                                                 ##"<<endl;
	cout << "#####################################################################"<<endl;
	cout << "## This program is free software.                                  ##" << endl;
 	cout << "## This program is distributed in the hope that it will be useful, ##" << endl;
 	cout << "## but WITHOUT ANY WARRANTY; without even the implied warranty of  ##" << endl;
 	cout << "## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.            ##" << endl;
 	cout << "#####################################################################" << endl;
	cout << endl;

	printf("Cleaning temporary files...");
	fflush(stdout);
	system("rmdir /S /Q temp");
	system("mkdir temp");
	printf(" done\n\n");


	nombre_images_par_page = 0;
	int ancienne_nb_page=0;
	while (nombre_images_par_page != 1 && nombre_images_par_page != 2)
	{
		std::cout << "Number of pages per image (1 or 2): ";
		cin>>nombre_images_par_page;
		cin.ignore(numeric_limits<streamsize>::max(), '\n');
	}
	ancienne_nb_page = nombre_images_par_page;

	int multipage__sens_page=0;
	if (nombre_images_par_page==2)
	{
		cout<<"Page layout :"<<endl<<"\t1: Right to Left"<<endl<<"\t2: Left to Right"<<endl;
		while (multipage__sens_page != 1 && multipage__sens_page != 2)
		{
			std::cout << "\tChoose: ";
			cin>>multipage__sens_page;
			cin.ignore(numeric_limits<streamsize>::max(), '\n');
		}
	}

	char auto_crop_on;
	std::cout << "Activate auto-crop (Y/N): ";
	cin>>auto_crop_on;
	cin.ignore(numeric_limits<streamsize>::max(), '\n');
	while(auto_crop_on != 'y' && auto_crop_on != 'Y' && auto_crop_on != 'n' && auto_crop_on != 'N')
	{
		std::cout << "Activate auto-crop (Y/N): ";
		cin>>auto_crop_on;
		cin.ignore(numeric_limits<streamsize>::max(), '\n');
	}
	if (auto_crop_on == 'y')
	{
		auto_crop_on = 'Y';
	}

	int mode_cropping_top_bottom=0;
	float coefficient_cropping_haut;
	float coefficient_cropping_bas;
	if (auto_crop_on == 'Y')
	{
		cout<<"Top/Bottom cropping mode:"<<endl<<"\t1: Light"<<endl<<"\t2: Medium"<<endl<<"\t3: Strong"<<endl;
		while (mode_cropping_top_bottom != 1 && mode_cropping_top_bottom != 2 && mode_cropping_top_bottom != 3)
		{
			std::cout << "\tCropping mode: ";
			cin>>mode_cropping_top_bottom;
			cin.ignore(numeric_limits<streamsize>::max(), '\n');
		}
		if (mode_cropping_top_bottom == 1)
		{
			coefficient_cropping_haut = CROPPING_TOP_LIGHT;
			coefficient_cropping_bas = CROPPING_BOTTOM_LIGHT;
		}
		else if (mode_cropping_top_bottom == 2)
		{
			coefficient_cropping_haut = CROPPING_TOP_MEDIUM;
			coefficient_cropping_bas = CROPPING_BOTTOM_MEDIUM;
		} else
		{
			coefficient_cropping_haut = CROPPING_TOP_STRONG;
			coefficient_cropping_bas = CROPPING_BOTTOM_STRONG;
		}
	}

	int modele_ecran=0;
	cout<<"Screen size:"<<endl<<"\t1: Sony: "<<LARGEUR_SONY<<"x"<<HAUTEUR_SONY<<endl<<"\t2: Other (Samsung, ...): "<<LARGEUR_SAMSUNG<<"x"<<HAUTEUR_SAMSUNG<<endl;
	while (modele_ecran != MODELE_ECRAN_SONY && modele_ecran != MODELE_ECRAN_SAMSUNG)
	{
		std::cout << "\tScreen size: ";
		cin>>modele_ecran;
		cin.ignore(numeric_limits<streamsize>::max(), '\n');
	}


	int format_sortie=FORMAT_NON_SPECIFIE;
	if (format_sortie==FORMAT_NON_SPECIFIE)
	{
		std::cout << "Output file type\n\t1: EPUB\n\t2: LRF (Sony)\n";
		while (format_sortie != FORMAT_EPUB && format_sortie != FORMAT_LRF)
		{
			std::cout << "\tFile format: ";
			cin>>format_sortie;
			cin.ignore(numeric_limits<streamsize>::max(), '\n');
		}
	}

	char livre_titre[1000];
	std::cout << "Book title : ";
	cin.getline(livre_titre,512);


	char livre_auteur[1000];
	std::cout << "Book author : ";
	cin.getline(livre_auteur,512);


    //////////////////////////////////////////
    // List images in directory
    static List<char*> liste_fichiers_jpg;
	char chaine[10000];

	for (int i=0 ; i<99999 ; i++)	arborescence[i].hierarchie=0;

	lister_repertoire(&liste_fichiers_jpg, ".\\images", 1);
	cout<<endl<<liste_fichiers_jpg.getSize()<<" pictures in directory."<<endl;

	//////////////////////////////////////////
	    // Load pictures & Cropping
	    Image image_origine;
		List<int> liste_images_finales_largeur;
		List<int> liste_images_finales_hauteur;
		int largeur_image;
		int hauteur_image;

	    liste_fichiers_jpg.moveToHead();

	    // If no picture in directory
	    if (liste_fichiers_jpg.getSize() == 0)
	    {
	        printf("There is no JPG pictures in directory.\n");
	        printf("\nPress <Enter> to exit.\n");
	        getchar();
	        exit(1);
	    }

	    printf("\nLoading images...\n");

		int rotation_page = 0;
		int page_verticale_ne_pas_couper;

	    for (int i=0 ; i<liste_fichiers_jpg.getSize() ; i++)
	    {
	        char *nom_image = *liste_fichiers_jpg.next();

			// conversion JPG ==> BMP 24 bits
			sprintf(chaine, "ImageMagick\\convert \"%s.jpg\" \"%s.jpg\" \"%s.jpg\" -combine temp\\temp.bmp", nom_image, nom_image, nom_image);
			//std::cout<<chaine<<"\n";
			system(chaine);

	        Image_Anomalie ano;
	        sprintf(chaine, "temp\\temp.bmp");
	        ano = image_origine.ChargeBMP(chaine);
			std::cout<<"Working with picture \""<<nom_image<<"\" : "<<image_origine.width<<" x "<<image_origine.height<<"\n";
			std::cout<<"\tLoading : "<<image_origine.LibelleAnomalie(ano)<<endl;

			// picture normalisation
			int val_min=255;
			int val_max=0;
			int val_pixel;
			// search for min et max
			for (int x=0 ; x<image_origine.width ; x++)
			{
				for (int y=0 ; y<image_origine.height ; y++)
				{
					val_pixel = (image_origine.pixel(x, y, 0)+image_origine.pixel(x, y, 1)+image_origine.pixel(x, y, 2))/3;
					if (val_pixel < val_min)	val_min = val_pixel;
					if (val_pixel > val_max)	val_max = val_pixel;
				}
			}
			cout<<"\tNormalization ("<<val_min<<"/"<<val_max<<"): ";
			if ((val_min == 0) && (val_max == 255))
			{
				cout<<"unnecessary."<<endl;
			}
			else
			{
				double scaleval = 255.0/double(val_max-val_min);
				for (int x=0 ; x<image_origine.width ; x++)
				{
					for (int y=0 ; y<image_origine.height ; y++)
					{
						val_pixel = (image_origine.pixel(x, y, 0)+image_origine.pixel(x, y, 1)+image_origine.pixel(x, y, 2))/3;
						val_pixel = int((double(val_pixel-val_min))*scaleval);
						image_origine.set_pixel(x, y, 0, val_pixel);
						image_origine.set_pixel(x, y, 1, val_pixel);
						image_origine.set_pixel(x, y, 2, val_pixel);
					}
				}
				cout<<"done."<<endl;
			}
			// end of normalisation

			//variables
			int fin;
			unsigned long int seuil;

			int colonnes_supprimees_droite = 0;
			int lignes_supprimees_haut = 0;
			int colonnes_supprimees_gauche = 0;
			int lignes_supprimees_bas = 0;

			if (auto_crop_on == 'Y')
			{
				// Cropping top
				std::cout<<"\tCropping top, ";
				fin=0;
				seuil = (unsigned long)((image_origine.width * 255 * 3)*coefficient_cropping_haut/(float)100.0);
				for (int y=0 ; (y<image_origine.height)&&(fin==0) ; y++)
				{
					unsigned long int total_ligne=0;
					unsigned long int total_pixel=0;
					for (int x=0 ; x<image_origine.width ; x++)
					{
						total_pixel = (255-image_origine.pixel(x, y, 0)) + (255-image_origine.pixel(x, y, 1)) + (255-image_origine.pixel(x, y, 2));
						if (total_pixel > 128*3)
							total_ligne += (255-image_origine.pixel(x, y, 0)) + (255-image_origine.pixel(x, y, 1)) + (255-image_origine.pixel(x, y, 2));
					}
					if (total_ligne > seuil)
					{
						fin=1;
					} else
					{
						lignes_supprimees_haut++;
					}
				}
				if (lignes_supprimees_haut == image_origine.height)
					lignes_supprimees_haut = 0;
				std::cout<<lignes_supprimees_haut<<" rows removed.\n";

				// Cropping bottom
				std::cout<<"\tCropping bottom, ";
				seuil = (unsigned long)((image_origine.width * 255 * 3)*coefficient_cropping_bas/(float)100.0);
				fin=0;
				for (int y=image_origine.height-1 ; (y>=0)&&(fin==0) ; y--)
				{
					unsigned long int total_ligne=0;
					unsigned long int total_pixel=0;
					for (int x=0 ; x<image_origine.width ; x++)
					{
						total_pixel = (255-image_origine.pixel(x, y, 0)) + (255-image_origine.pixel(x, y, 1)) + (255-image_origine.pixel(x, y, 2));
						if (total_pixel > 128*3)
							total_ligne += (255-image_origine.pixel(x, y, 0)) + (255-image_origine.pixel(x, y, 1)) + (255-image_origine.pixel(x, y, 2));
					}
					if (total_ligne > seuil)
					{
						fin=1;
					} else
					{
						lignes_supprimees_bas++;
					}
				}
				if (lignes_supprimees_bas == image_origine.height)
					lignes_supprimees_bas = 0;
				std::cout<<lignes_supprimees_bas<<" rows removed.\n";


				// Cropping left
				std::cout<<"\tCropping left, ";
				fin=0;
				seuil = (image_origine.height * 255 * 3)*1/100;
				for (int x=0 ; (x<image_origine.width)&&(fin==0) ; x++)
				{
					unsigned long int total_colonne=0;
					unsigned long int total_pixel=0;
					for (int y=0 ; y<image_origine.height ; y++)
					{
						total_pixel = (255-image_origine.pixel(x, y, 0)) + (255-image_origine.pixel(x, y, 1)) + (255-image_origine.pixel(x, y, 2));
						if (total_pixel > 128*3)
							total_colonne += (255-image_origine.pixel(x, y, 0)) + (255-image_origine.pixel(x, y, 1)) + (255-image_origine.pixel(x, y, 2));
					}
					if (total_colonne > seuil)
					{
						fin=1;
					} else
					{
						colonnes_supprimees_gauche++;
					}
				}
				if (colonnes_supprimees_gauche == image_origine.width)
					colonnes_supprimees_gauche = 0;
				std::cout<<colonnes_supprimees_gauche<<" columns removed.\n";


				// Cropping right
				std::cout<<"\tCropping right, ";
				fin=0;
				seuil = (image_origine.height * 255 * 3)*1/100;
				for (int x=image_origine.width-1 ; (x>=0)&&(fin==0) ; x--)
				{
					unsigned long int total_colonne=0;
					unsigned long int total_pixel=0;
					for (int y=0 ; y<image_origine.height ; y++)
					{
						total_pixel = (255-image_origine.pixel(x, y, 0)) + (255-image_origine.pixel(x, y, 1)) + (255-image_origine.pixel(x, y, 2));
						if (total_pixel > 128*3)
							total_colonne += (255-image_origine.pixel(x, y, 0)) + (255-image_origine.pixel(x, y, 1)) + (255-image_origine.pixel(x, y, 2));
					}
					if (total_colonne > seuil)
					{
						fin=1;
					} else
					{
						colonnes_supprimees_droite++;
					}
				}
				if (colonnes_supprimees_droite == image_origine.width)
					colonnes_supprimees_droite = 0;
				std::cout<<colonnes_supprimees_droite<<" columns removed.\n";
			}


			largeur_image = image_origine.width-colonnes_supprimees_droite-colonnes_supprimees_gauche;
			hauteur_image = image_origine.height-lignes_supprimees_haut-lignes_supprimees_bas;

			if (nombre_images_par_page == 1)
			{
				if (largeur_image > hauteur_image)
				{
					rotation_page = 1;
					int temp;
					temp = largeur_image;
					largeur_image = hauteur_image;
					hauteur_image = temp;
				}
				else rotation_page = 0;
			}
			else
			{
				rotation_page = 0;
			}

			std::cout<<"\tNew picture size: "<< largeur_image <<" x "<< hauteur_image <<"\n";

			//calculation of new picture size
			//computing final picture size to keep aspect ratio
			int largeur_ratio;
			int hauteur_ratio;
			float temp_a, temp_b;
			if (modele_ecran==MODELE_ECRAN_SONY)
			{
				temp_a = LARGEUR_SONY;
				temp_b = HAUTEUR_SONY;
			} else
			{
				temp_a = LARGEUR_SAMSUNG;
				temp_b = HAUTEUR_SAMSUNG;
			}
			if (((float)temp_a/(float)temp_b) > ((float)largeur_image/(float)hauteur_image))
			{
				ancienne_nb_page = nombre_images_par_page;
				nombre_images_par_page = 1;
				if (modele_ecran==MODELE_ECRAN_SONY)
						hauteur_ratio = HAUTEUR_SONY;
				else	hauteur_ratio = HAUTEUR_SAMSUNG;
				largeur_ratio = (int)((float)hauteur_ratio / (float)hauteur_image * (float)largeur_image/(float)nombre_images_par_page);
				page_verticale_ne_pas_couper = 1;
				nombre_images_par_page = ancienne_nb_page;
			}
			else
			{
				if (modele_ecran==MODELE_ECRAN_SONY)
						largeur_ratio = LARGEUR_SONY;
				else	largeur_ratio = LARGEUR_SAMSUNG;
				hauteur_ratio = (int)((float)largeur_ratio / ((float)largeur_image/(float)nombre_images_par_page) * (float)hauteur_image);
				page_verticale_ne_pas_couper = 0;
			}

			// generate pnm picture
			int valeur;
		    std::ofstream image_pnm("temp\\temp.pnm");
			if (page_verticale_ne_pas_couper==1)
			{
				image_pnm << "P2\n"<< largeur_image <<" "<< hauteur_image <<"\n255\n";
				if (rotation_page == 1)
				{
					int temp;
					temp = largeur_image;
					largeur_image = hauteur_image;
					hauteur_image = temp;
					for (int x=largeur_image-1 ; x>=0 ; x--)
					{
						for (int y=0 ; y<hauteur_image ; y++)
						{
							valeur = (image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0)
							+ image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0)
							+ image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0) ) /3;
							image_pnm << valeur << " ";
						}
						image_pnm << "\n";
					}

					temp = largeur_image;
					largeur_image = hauteur_image;
					hauteur_image = temp;
				}
				else
				{
					for (int y=0 ; y<hauteur_image ; y++)
					{
						for (int x=0 ; x<largeur_image ; x++)
						{
							valeur = (image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0)
							+ image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0)
							+ image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0) ) /3;
							image_pnm << valeur << " ";
						}
						image_pnm << "\n";
					}
				}
			}
			else
			{
				image_pnm << "P2\n"<< largeur_image/nombre_images_par_page <<" "<< hauteur_image <<"\n255\n";
				if (rotation_page == 1)
				{
					int temp;
					temp = largeur_image;
					largeur_image = hauteur_image;
					hauteur_image = temp;
					for (int x=largeur_image-1 ; x>=0 ; x--)
					{
						for (int y=0 ; y<hauteur_image ; y++)
						{
							valeur = (image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0)
							+ image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0)
							+ image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0) ) /3;
							image_pnm << valeur << " ";
						}
						image_pnm << "\n";
					}

					temp = largeur_image;
					largeur_image = hauteur_image;
					hauteur_image = temp;
				}
				else
				{
					for (int y=0 ; y<hauteur_image ; y++)
					{
						for (int x=0 ; x<largeur_image/nombre_images_par_page ; x++)
						{
							valeur = (image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0)
							+ image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0)
							+ image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0) ) /3;
							image_pnm << valeur << " ";
						}
						image_pnm << "\n";
					}
				}
			}
			image_pnm.close();

			if (modele_ecran==MODELE_ECRAN_SONY)
					sprintf(chaine, "NetPBM\\pnmscale -xsize=%i -ysize=%i temp\\temp.pnm | NetPBM\\pgmenhance -7 | NetPBM\\pnmremap -mapfile=NetPBM\\palette.pgm | NetPBM\\pnmpad -white -width %i -height %i > temp\\temp2.pnm", largeur_ratio, hauteur_ratio, LARGEUR_SONY, HAUTEUR_SONY);
			else	sprintf(chaine, "NetPBM\\pnmscale -xsize=%i -ysize=%i temp\\temp.pnm | NetPBM\\pgmenhance -7 | NetPBM\\pnmremap -mapfile=NetPBM\\palette.pgm | NetPBM\\pnmpad -white -width %i -height %i > temp\\temp2.pnm", largeur_ratio, hauteur_ratio, LARGEUR_SAMSUNG, HAUTEUR_SAMSUNG);
			system(chaine);

			if (multipage__sens_page==2)
			{	// japonese reading page order
				sprintf(chaine, "NetPBM\\pnmtopng temp\\temp2.pnm > temp\\%05d.png", (i*nombre_images_par_page)+1);
			} else
			{ // standard reading page order
				sprintf(chaine, "NetPBM\\pnmtopng temp\\temp2.pnm > temp\\%05d.png", (i*nombre_images_par_page));
			}

			system(chaine);

			sprintf(chaine, "temp\\%s.bmp", nom_image);
			remove(chaine);

			if ((nombre_images_par_page == 2) && (page_verticale_ne_pas_couper == 0))
			{
				// working with second picture
				// generate pnm picture
				int valeur;
			    std::ofstream image_pnm("temp\\temp.pnm");
				image_pnm << "P2\n"<< largeur_image/nombre_images_par_page <<" "<< hauteur_image <<"\n255\n";
				for (int y=0 ; y<hauteur_image ; y++)
				{
					for (int x=largeur_image/nombre_images_par_page ; x<2*(largeur_image/nombre_images_par_page) ; x++)
					{
						valeur = (image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0)
						+ image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0)
						+ image_origine.pixel(x+colonnes_supprimees_gauche, y+lignes_supprimees_haut,0) ) /3;
						image_pnm << valeur << " ";
					}
					image_pnm << "\n";
				}
				image_pnm.close();

				//computing new picture size, keeping aspect ratio
				int largeur_ratio;
				int hauteur_ratio;
				if (modele_ecran==MODELE_ECRAN_SONY)
				{
					if (((float)LARGEUR_SONY/(float)HAUTEUR_SONY) > ((float)largeur_image/(float)nombre_images_par_page/(float)hauteur_image))
					{
						hauteur_ratio = HAUTEUR_SONY;
						largeur_ratio = (int)((float)hauteur_ratio / (float)hauteur_image * (float)largeur_image/(float)nombre_images_par_page);
					}
					else
					{
						largeur_ratio = LARGEUR_SONY;
						hauteur_ratio = (int)((float)largeur_ratio / ((float)largeur_image/(float)nombre_images_par_page) * (float)hauteur_image);
					}
				} else
				{	// SAMSUNG
					if (((float)LARGEUR_SAMSUNG/(float)HAUTEUR_SAMSUNG) > ((float)largeur_image/(float)nombre_images_par_page/(float)hauteur_image))
					{
						hauteur_ratio = HAUTEUR_SAMSUNG;
						largeur_ratio = (int)((float)hauteur_ratio / (float)hauteur_image * (float)largeur_image/(float)nombre_images_par_page);
					}
					else
					{
						largeur_ratio = LARGEUR_SAMSUNG;
						hauteur_ratio = (int)((float)largeur_ratio / ((float)largeur_image/(float)nombre_images_par_page) * (float)hauteur_image);
					}
				}

				if (modele_ecran==MODELE_ECRAN_SONY)
						sprintf(chaine, "NetPBM\\pnmscale -xsize=%i -ysize=%i temp\\temp.pnm | NetPBM\\pgmenhance -7 | NetPBM\\pnmremap -mapfile=NetPBM\\palette.pgm | NetPBM\\pnmpad -white -width %i -height %i > temp\\temp2.pnm", largeur_ratio, hauteur_ratio, LARGEUR_SONY, HAUTEUR_SONY);
				else 	sprintf(chaine, "NetPBM\\pnmscale -xsize=%i -ysize=%i temp\\temp.pnm | NetPBM\\pgmenhance -7 | NetPBM\\pnmremap -mapfile=NetPBM\\palette.pgm | NetPBM\\pnmpad -white -width %i -height %i > temp\\temp2.pnm", largeur_ratio, hauteur_ratio, LARGEUR_SAMSUNG, HAUTEUR_SAMSUNG);
				system(chaine);

				if (multipage__sens_page==2)
				{	// japanese reading
					sprintf(chaine, "NetPBM\\pnmtopng temp\\temp2.pnm > temp\\%05d.png", (i*nombre_images_par_page));
				} else
				{ // standard reading
					sprintf(chaine, "NetPBM\\pnmtopng temp\\temp2.pnm > temp\\%05d.png", (i*nombre_images_par_page)+1);
				}
				system(chaine);

				sprintf(chaine, "temp\\%s.bmp", nom_image);
				remove(chaine);
			}

	    } // fin de loop sur les images


		if (format_sortie == FORMAT_LRF)
		{
			system("copy book_thumb.gif temp\\book_thumb.gif > temp\\temp.txt");

			// creating temp.lrs file
			std::ofstream fichier_lrs("temp\\temp.lrs");
			fichier_lrs << "<BBeBXylog version=\"1.0\"><Property/><BookInformation><Info version=\"1.0\"><BookInfo>\n";
			fichier_lrs << "<Title reading=\"\">" << livre_titre << "</Title>\n";
			fichier_lrs << "<Author reading=\"\">" << livre_auteur << "</Author>\n";
			fichier_lrs << "<BookID>FB_0000000000001</BookID>\n";
			fichier_lrs << "<Publisher reading=\"\"></Publisher>\n";
			fichier_lrs << "<Label reading=\"\"></Label>\n";
			fichier_lrs << "<Category></Category>\n";
			fichier_lrs << "<Classification></Classification>\n";
			fichier_lrs << "<FreeText></FreeText>\n";
			fichier_lrs << "</BookInfo><DocInfo>\n";
			fichier_lrs << "<CThumbnail file=\"book_thumb.gif\"/>\n";
			fichier_lrs << "<Language>fr</Language>\n";
			fichier_lrs << "<Creator></Creator>\n";
			fichier_lrs << "<CreationDate></CreationDate>\n";
			fichier_lrs << "<Producer>Scan2LRF</Producer>\n";
			fichier_lrs << "<SumPage>" << liste_fichiers_jpg.getSize() << "</SumPage>\n";
			fichier_lrs << "</DocInfo></Info>\n";
			fichier_lrs << "<TOC>\n";


			char compteur[10];
			for (int i=0 ; i<liste_fichiers_jpg.getSize()*nombre_images_par_page ; i++)	//for (int i=0 ; i<99999 ; i++)
			{
				if(arborescence[i].hierarchie != 0)
				{
					sprintf(compteur, "%05d", arborescence[i].numero_image);
					fichier_lrs << "<TocLabel refobj=\"2" << compteur << "\" refpage=\"1" << compteur <<"\">" << arborescence[i].nom_lien << "</TocLabel>" <<endl;
				}
			}

			fichier_lrs << "</TOC>\n";
			fichier_lrs << "</BookInformation><Main>\n";

			for (int i=0 ; i<liste_fichiers_jpg.getSize()*nombre_images_par_page ; i++)
			{
				sprintf(compteur, "%05d", i);
				sprintf(chaine, "temp\\%s.png", compteur);
				if (FileExists(chaine))
					fichier_lrs << "<Page pagestyle=\"77\" objid=\"1" << compteur << "\" objlabel=\"Page.1" << compteur << "\"><BlockSpace xspace=\"0\" yspace=\"0\"/><ImageBlock x0=\"" << 0 << "\" y0=\"" << 0 << "\" x1=\"" << 600 << "\" y1=\"" << 800 << "\" xsize=\"" << 637 << "\" ysize=\"" << 849 << "\" refstream=\"3" << compteur << "\" blockwidth=\"" << 600 << "\" blockheight=\"" << 768 << "\" topskip=\"0\" footskip=\"0\" sidemargin=\"0\" blockstyle=\"86\" objid=\"2" << compteur << "\" objlabel=\"Block.2" << compteur << "\"></ImageBlock></Page>\n";
			}

			fichier_lrs << "</Main><Template version=\"1.0\"></Template><Style>\n";
			fichier_lrs << "<BookStyle stylelabel=\"58\" objid=\"58\"><SetDefault rubyalign=\"center\" rubyadjust=\"none\" rubyoverhang=\"none\" empdotsposition=\"before\" emplineposition=\"before\" emplinetype=\"solid\" setwaitprop=\"noreplay\"/>\n";
			fichier_lrs << "<BookSetting bindingdirection=\"Lr\" dpi=\"1600\" screenwidth=\"600\" screenheight=\"800\" colordepth=\"24\"/></BookStyle>\n";
			fichier_lrs << "<PageStyle stylelabel=\"77\" topmargin=\"0\" headheight=\"0\" headsep=\"0\" oddsidemargin=\"0\" textheight=\"800\" textwidth=\"600\" footspace=\"0\" evensidemargin=\"0\" footheight=\"0\" layout=\"LrTb\" bgimagemode=\"fix\" pageposition=\"any\" setwaitprop=\"noreplay\" setemptyview=\"empty\" objid=\"77\"/>\n";
			fichier_lrs << "<BlockStyle stylelabel=\"86\" bgimagemode=\"fix\" framemode=\"square\" blockwidth=\"600\" blockheight=\"800\" blockrule=\"horz-adjustable\" layout=\"LrTb\" framewidth=\"0\" framecolor=\"0x00000000\" topskip=\"0\" sidemargin=\"0\" footskip=\"0\" objid=\"86\"/>\n";
			fichier_lrs << "</Style><Objects>\n";

			for (int i=0 ; i<liste_fichiers_jpg.getSize()*nombre_images_par_page ; i++)
			{
				sprintf(compteur, "%05d", i);
				sprintf(chaine, "temp\\%s.png", compteur);
				if (FileExists(chaine))
					fichier_lrs << "<ImageStream encoding=\"PNG\" file=\"" << compteur << ".png\" objid=\"3" << compteur << "\" imagestreamlabel=\"ImageStream.3" << compteur << "\"/>\n";
			}

			fichier_lrs << "</Objects></BBeBXylog>\n";

			fichier_lrs.close();

			chdir("temp");

			// remove accentuated chars
			char nouveau_char;

			for(int i = 0; i<strlen(livre_titre) ; i++)
			{
				nouveau_char=livre_titre[i];
				switch(livre_titre[i])
				{
					case 0x82: nouveau_char='e'; break;
					case 0x85: nouveau_char='a'; break;
					case 0x87: nouveau_char='c'; break;
					case 0x8a: nouveau_char='e'; break;
					case 0x88: nouveau_char='e'; break;
					case 0x89: nouveau_char='e'; break;
					case 0xd3: nouveau_char='e'; break;
					case 0xd2: nouveau_char='e'; break;
					case 0x97: nouveau_char='u'; break;
					case 0x81: nouveau_char='u'; break;
					case 0x96: nouveau_char='u'; break;
					case 0xea: nouveau_char='u'; break;
					case 0x9a: nouveau_char='u'; break;
					case 0x83: nouveau_char='a'; break;
					case 0x84: nouveau_char='a'; break;
					case 0xb6: nouveau_char='a'; break;
					case 0x8e: nouveau_char='a'; break;
				}
				livre_titre[i]=nouveau_char;
			}

			sprintf(chaine, "..\\lrs2lrf\\lrs2lrf.exe temp.lrs \"..\\%s.lrf\"", livre_titre);
			system(chaine);

		// cleaning temporary files
		printf("Cleaning temporary files...");
		fflush(stdout);
		chdir("..\\");
		system("rmdir /S /Q temp");
		system("mkdir temp");
		printf(" done\n\n");

		cout << "File \"" << livre_titre << ".lrf\" created.\n";

	}
	else
	{	//format EPUB
		system("mkdir temp\\OEBPS");
		system("mkdir temp\\OEBPS\\images");

		std::ofstream fichier_page;
		int numero_chapitre_actuel = 1;

		int nombre_images=0;
		char compteur[10];

		for (int i=0 ; i<liste_fichiers_jpg.getSize()*nombre_images_par_page ; i++)	//for (int i=0 ; i<99999 ; i++)
		{
			sprintf(chaine, "temp\\OEBPS\\%05d.xhtml", i);
			fichier_page.open(chaine);

			fichier_page << "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>" << endl;
			fichier_page << "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">" << endl;
			fichier_page << "<html xmlns=\"http://www.w3.org/1999/xhtml\">" << endl;
			fichier_page << "<head>" << endl;

			if (arborescence[i].hierarchie != 0)
			{
				fichier_page << "<title>" << arborescence[i].nom_lien << "</title>" << endl;
			}
			fichier_page << "<link href=\"stylesheet.css\" type=\"text/css\" rel=\"stylesheet\" />" << endl;
			fichier_page << "<link rel=\"stylesheet\" type=\"application/vnd.adobe-page-template+xml\" href=\"page-template.xpgt\"/>" << endl;
			fichier_page << "</head>" << endl;
			fichier_page << "<body>" << endl;

			sprintf(compteur, "%05d", i);
			sprintf(chaine, "temp\\%s.png", compteur);
			if (FileExists(chaine))
			{
				nombre_images++;
				fichier_page << "<p class=\"CI\">";
				fichier_page << "<img src=\"images/" << compteur << ".png\"/>";
				fichier_page << "</p>" << endl;
				sprintf(chaine, "move temp\\%s.png temp\\OEBPS\\images\\%s.png", compteur, compteur);
				system(chaine);
			}
			fichier_page << "</body>" << endl;
			fichier_page << "</html>" << endl;
			fichier_page.close();
		}

		// creating file "contents.opf"
		std::ofstream fichier_content("temp\\OEBPS\\content.opf");
		fichier_content << "<?xml version=\"1.0\"?>"<<endl;
		fichier_content << "<package version=\"2.0\" xmlns=\"http://www.idpf.org/2007/opf\""<<endl;
		fichier_content << "         unique-identifier=\"BookId\">"<<endl;
		fichier_content << "<metadata xmlns:dc=\"http://purl.org/dc/elements/1.1/\""<<endl;
		fichier_content << "           xmlns:opf=\"http://www.idpf.org/2007/opf\">"<<endl;
		fichier_content << "  <dc:title>"<< livre_titre << "</dc:title>"<<endl;
		fichier_content << "  <dc:creator opf:role=\"aut\">" << livre_auteur << "</dc:creator>"<<endl;
		fichier_content << "  <dc:language></dc:language>"<<endl;
		fichier_content << "  <dc:rights></dc:rights>"<<endl;
		fichier_content << "  <dc:publisher></dc:publisher>"<<endl;
		srand ( time(NULL) );
		int rand1 = rand();
		int rand2 = rand();
		int rand3 = rand();
		fichier_content << "  <dc:identifier id=\"BookId\">urn:uuid:" << rand1 << rand2 << rand3 <<"</dc:identifier>"<<endl;
		fichier_content << "</metadata>"<<endl;
		fichier_content << "<manifest>"<<endl;
		fichier_content << "  <item id=\"ncx\" href=\"toc.ncx\" media-type=\"application/x-dtbncx+xml\" />"<<endl;
		fichier_content << "  <item id=\"style\" href=\"stylesheet.css\" media-type=\"text/css\" />"<<endl;
		fichier_content << "  <item id=\"pagetemplate\" href=\"page-template.xpgt\" media-type=\"application/vnd.adobe-page-template+xml\" />"<<endl;
		for (int i=0 ; i<liste_fichiers_jpg.getSize()*nombre_images_par_page ; i++)	//for (int i=0 ; i<99999 ; i++)
		{
			sprintf(compteur, "%05d", i);
			sprintf(chaine, "temp\\OEBPS\\images\\%s.png", compteur);
			if (FileExists(chaine))
			{
				sprintf(chaine, "%05d.xhtml", i);
				fichier_content << "  <item id=\"" << compteur << "\" href=\"" << chaine << "\" media-type=\"application/xhtml+xml\" />"<<endl;

			}
		}
		fichier_content << "</manifest>"<<endl;
		fichier_content << "<spine toc=\"ncx\">"<<endl;
		for (int i=0 ; i<liste_fichiers_jpg.getSize()*nombre_images_par_page ; i++)	//for (int i=0 ; i<99999 ; i++)
		{
			sprintf(compteur, "%05d", i);
			fichier_content << "  <itemref idref=\"" << compteur << "\" />"<<endl;
		}
		fichier_content << "</spine>"<<endl;
		fichier_content << "</package>"<<endl;
		fichier_content.close();

		// crating "toc.ncx" file
		std::ofstream fichier_toc("temp\\OEBPS\\toc.ncx");
		fichier_toc << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"<<endl;
		fichier_toc << "<ncx xmlns=\"http://www.daisy.org/z3986/2005/ncx/\" version=\"2005-1\">"<<endl;
		fichier_toc << "  <head>"<<endl;
		fichier_toc << "    <meta name=\"dtb:uid\" content=\"" << rand1 << rand2 << rand3 << "\"/>"<<endl;
		fichier_toc << "    <meta name=\"dtb:depth\" content=\"1\"/>"<<endl;
		fichier_toc << "    <meta name=\"dtb:totalPageCount\" content=\"" << nombre_images << "\"/>"<<endl;
		fichier_toc << "    <meta name=\"dtb:maxPageNumber\" content=\"" << nombre_images << "\"/>"<<endl;
		fichier_toc << "  </head>"<<endl;
		fichier_toc << "  <docTitle>"<<endl;
		fichier_toc << "    <text>" << livre_titre << "</text>"<<endl;
		fichier_toc << "  </docTitle>"<<endl;
		fichier_toc << "  <navMap>"<<endl;
		if (arborescence[0].hierarchie == 0)
		{
			fichier_toc << "    <navPoint id=\"00000\" playOrder=\"1\">"<<endl;
			fichier_toc << "      <navLabel>"<<endl;
			fichier_toc << "        <text>Introduction</text>"<<endl;
			fichier_toc << "      </navLabel>"<<endl;
			fichier_toc << "      <content src=\"00000.xhtml\"/>"<<endl;
			fichier_toc << "    </navPoint>"<<endl;
		}
		numero_chapitre_actuel=1;
		for (int i=0 ; i<liste_fichiers_jpg.getSize()*nombre_images_par_page ; i++)	//for (int i=0 ; i<99999 ; i++)
		{
			if(arborescence[i].hierarchie != 0)
			{
				sprintf(chaine, "%05d.xhtml", i);
				sprintf(compteur, "%05d", i);
				fichier_toc << "    <navPoint id=\""<< compteur << "\" playOrder=\"" << numero_chapitre_actuel << "\">"<<endl;
				fichier_toc << "      <navLabel>"<<endl;
				fichier_toc << "        <text>" << arborescence[i].nom_lien << "</text>"<<endl;
				fichier_toc << "      </navLabel>"<<endl;
				fichier_toc << "      <content src=\"" << chaine << "\"/>"<<endl;
				fichier_toc << "    </navPoint>"<<endl;
				numero_chapitre_actuel++;
			}
		}
		fichier_toc << "  </navMap>"<<endl;
		fichier_toc << "</ncx>"<<endl;
		fichier_toc.close();

		// crating XPGT file
		std::ofstream fichier_xpgt("temp\\OEBPS\\page-templace.xpgt");
		fichier_xpgt << "<ade:template xmlns=\"http://www.w3.org/1999/xhtml\" xmlns:ade=\"http://ns.adobe.com/2006/ade\"" << endl;
		fichier_xpgt << "		 xmlns:fo=\"http://www.w3.org/1999/XSL/Format\">" << endl;

		fichier_xpgt << "  <fo:layout-master-set>" << endl;
		fichier_xpgt << "   <fo:simple-page-master master-name=\"single_column\">" << endl;
		fichier_xpgt << "		<fo:region-body margin-bottom=\"3pt\" margin-top=\"0.5em\" margin-left=\"3pt\" margin-right=\"3pt\"/>" << endl;
		fichier_xpgt << "    </fo:simple-page-master>" << endl;

		fichier_xpgt << "    <fo:simple-page-master master-name=\"single_column_head\">" << endl;
		fichier_xpgt << "		<fo:region-before extent=\"8.3em\"/>" << endl;
		fichier_xpgt << "		<fo:region-body margin-bottom=\"3pt\" margin-top=\"6em\" margin-left=\"3pt\" margin-right=\"3pt\"/>" << endl;
		fichier_xpgt << "    </fo:simple-page-master>" << endl;

		fichier_xpgt << "    <fo:simple-page-master master-name=\"two_column\"	margin-bottom=\"0.5em\" margin-top=\"0.5em\" margin-left=\"0.5em\" margin-right=\"0.5em\">" << endl;
		fichier_xpgt << "		<fo:region-body column-count=\"2\" column-gap=\"10pt\"/>" << endl;
		fichier_xpgt << "    </fo:simple-page-master>" << endl;

		fichier_xpgt << "    <fo:simple-page-master master-name=\"two_column_head\" margin-bottom=\"0.5em\" margin-left=\"0.5em\" margin-right=\"0.5em\">" << endl;
		fichier_xpgt << "		<fo:region-before extent=\"8.3em\"/>" << endl;
		fichier_xpgt << "		<fo:region-body column-count=\"2\" margin-top=\"6em\" column-gap=\"10pt\"/>" << endl;
		fichier_xpgt << "    </fo:simple-page-master>" << endl;

		fichier_xpgt << "    <fo:simple-page-master master-name=\"three_column\" margin-bottom=\"0.5em\" margin-top=\"0.5em\" margin-left=\"0.5em\" margin-right=\"0.5em\">" << endl;
		fichier_xpgt << "		<fo:region-body column-count=\"3\" column-gap=\"10pt\"/>" << endl;
		fichier_xpgt << "    </fo:simple-page-master>" << endl;

		fichier_xpgt << "    <fo:simple-page-master master-name=\"three_column_head\" margin-bottom=\"0.5em\" margin-top=\"0.5em\" margin-left=\"0.5em\" margin-right=\"0.5em\">" << endl;
		fichier_xpgt << "		<fo:region-before extent=\"8.3em\"/>" << endl;
		fichier_xpgt << "		<fo:region-body column-count=\"3\" margin-top=\"6em\" column-gap=\"10pt\"/>" << endl;
		fichier_xpgt << "    </fo:simple-page-master>" << endl;

		fichier_xpgt << "    <fo:page-sequence-master>" << endl;
		fichier_xpgt << "        <fo:repeatable-page-master-alternatives>" << endl;
		fichier_xpgt << "            <fo:conditional-page-master-reference master-reference=\"three_column_head\" page-position=\"first\" ade:min-page-width=\"80em\"/>" << endl;
		fichier_xpgt << "            <fo:conditional-page-master-reference master-reference=\"three_column\" ade:min-page-width=\"80em\"/>" << endl;
		fichier_xpgt << "            <fo:conditional-page-master-reference master-reference=\"two_column_head\" page-position=\"first\" ade:min-page-width=\"50em\"/>" << endl;
		fichier_xpgt << "            <fo:conditional-page-master-reference master-reference=\"two_column\" ade:min-page-width=\"50em\"/>" << endl;
		fichier_xpgt << "            <fo:conditional-page-master-reference master-reference=\"single_column_head\" page-position=\"first\" />" << endl;
		fichier_xpgt << "            <fo:conditional-page-master-reference master-reference=\"single_column\"/>" << endl;
		fichier_xpgt << "        </fo:repeatable-page-master-alternatives>" << endl;
		fichier_xpgt << "    </fo:page-sequence-master>" << endl;

		fichier_xpgt << "  </fo:layout-master-set>" << endl;

		fichier_xpgt << "  <ade:style>" << endl;
		fichier_xpgt << "    <ade:styling-rule selector=\".title_box\" display=\"adobe-other-region\" adobe-region=\"xsl-region-before\"/>" << endl;
		fichier_xpgt << "  </ade:style>" << endl;

		fichier_xpgt << "</ade:template>" << endl;

		fichier_xpgt.close();


		// Crating CSS file
		std::ofstream fichier_css("temp\\OEBPS\\stylesheet.css");
		fichier_css << "/* Style Sheet */" << endl;
		fichier_css << "/* This defines styles and classes used in the book */" << endl;
		fichier_css << "body { margin-left: 0%; margin-right: 0%; margin-top: 0%; margin-bottom: 0%; text-align: justify; }" << endl;
		fichier_css << "pre { font-size: x-small; }" << endl;
		fichier_css << "h1 { text-align: center; }" << endl;
		fichier_css << "h2 { text-align: center; }" << endl;
		fichier_css << "h3 { text-align: center; }" << endl;
		fichier_css << "h4 { text-align: center; }" << endl;
		fichier_css << "h5 { text-align: center; }" << endl;
		fichier_css << "h6 { text-align: center; }" << endl;
		fichier_css << ".CI {" << endl;
		fichier_css << "	text-align:center;" << endl;
		fichier_css << "	margin-top:0px;" << endl;
		fichier_css << "	margin-bottom:0px;" << endl;
		fichier_css << "	padding:0px;" << endl;
		fichier_css << "	}" << endl;
		fichier_css << ".center   {text-align: center;}" << endl;
		fichier_css << ".smcap    {font-variant: small-caps;}" << endl;
		fichier_css << ".u        {text-decoration: underline;}" << endl;
		fichier_css << ".bold     {font-weight: bold;}" << endl;
		fichier_css.close();

		remove("temp\\modele.zip");
		system("copy modele.fmt temp\\modele.zip > temp\\temp.txt");
		chdir("temp");
		system("..\\infozip\\zip -0 -r modele.zip OEBPS > temp.txt");


		// remove accentuated chars
		char nouveau_char;

		for(int i = 0; i<strlen(livre_titre) ; i++)
		{
			nouveau_char=livre_titre[i];
			switch(livre_titre[i])
			{
				case 0x82: nouveau_char='e'; break;
				case 0x85: nouveau_char='a'; break;
				case 0x87: nouveau_char='c'; break;
				case 0x8a: nouveau_char='e'; break;
				case 0x88: nouveau_char='e'; break;
				case 0x89: nouveau_char='e'; break;
				case 0xd3: nouveau_char='e'; break;
				case 0xd2: nouveau_char='e'; break;
				case 0x97: nouveau_char='u'; break;
				case 0x81: nouveau_char='u'; break;
				case 0x96: nouveau_char='u'; break;
				case 0xea: nouveau_char='u'; break;
				case 0x9a: nouveau_char='u'; break;
				case 0x83: nouveau_char='a'; break;
				case 0x84: nouveau_char='a'; break;
				case 0xb6: nouveau_char='a'; break;
				case 0x8e: nouveau_char='a'; break;
			}
			livre_titre[i]=nouveau_char;
		}
		sprintf(chaine, "move modele.zip \"..\\%s.epub\"", livre_titre);
		system(chaine);

		// cleaning temporary files
		printf("Cleaning temporary files...");
		fflush(stdout);
		chdir("..\\");
		system("rmdir /S /Q temp");
		system("mkdir temp");
		printf(" done\n\n");


		cout << "File \"" << livre_titre << ".epub\" created.\n";
	}


	// cleaning files
	for (int i=0 ; i<99999 ; i++)
	{
		if (arborescence[i].hierarchie != 0)
		{
			delete(arborescence[i].nom_lien);
		}
	}

	cout << "Press any key to exit..." << endl;
	char b;
	cin.get(b);

	return 0;
}

/******************************************************************************
		    GNU GENERAL PUBLIC LICENSE
		       Version 2, June 1991

 Copyright (C) 1989, 1991 Free Software Foundation, Inc.
                       59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 Everyone is permitted to copy and distribute verbatim copies
 of this license document, but changing it is not allowed.

			    Preamble

  The licenses for most software are designed to take away your
freedom to share and change it.  By contrast, the GNU General Public
License is intended to guarantee your freedom to share and change free
software--to make sure the software is free for all its users.  This
General Public License applies to most of the Free Software
Foundation's software and to any other program whose authors commit to
using it.  (Some other Free Software Foundation software is covered by
the GNU Library General Public License instead.)  You can apply it to
your programs, too.

  When we speak of free software, we are referring to freedom, not
price.  Our General Public Licenses are designed to make sure that you
have the freedom to distribute copies of free software (and charge for
this service if you wish), that you receive source code or can get it
if you want it, that you can change the software or use pieces of it
in new free programs; and that you know you can do these things.

  To protect your rights, we need to make restrictions that forbid
anyone to deny you these rights or to ask you to surrender the rights.
These restrictions translate to certain responsibilities for you if you
distribute copies of the software, or if you modify it.

  For example, if you distribute copies of such a program, whether
gratis or for a fee, you must give the recipients all the rights that
you have.  You must make sure that they, too, receive or can get the
source code.  And you must show them these terms so they know their
rights.

  We protect your rights with two steps: (1) copyright the software, and
(2) offer you this license which gives you legal permission to copy,
distribute and/or modify the software.

  Also, for each author's protection and ours, we want to make certain
that everyone understands that there is no warranty for this free
software.  If the software is modified by someone else and passed on, we
want its recipients to know that what they have is not the original, so
that any problems introduced by others will not reflect on the original
authors' reputations.

  Finally, any free program is threatened constantly by software
patents.  We wish to avoid the danger that redistributors of a free
program will individually obtain patent licenses, in effect making the
program proprietary.  To prevent this, we have made it clear that any
patent must be licensed for everyone's free use or not licensed at all.

  The precise terms and conditions for copying, distribution and
modification follow.

		    GNU GENERAL PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. This License applies to any program or other work which contains
a notice placed by the copyright holder saying it may be distributed
under the terms of this General Public License.  The "Program", below,
refers to any such program or work, and a "work based on the Program"
means either the Program or any derivative work under copyright law:
that is to say, a work containing the Program or a portion of it,
either verbatim or with modifications and/or translated into another
language.  (Hereinafter, translation is included without limitation in
the term "modification".)  Each licensee is addressed as "you".

Activities other than copying, distribution and modification are not
covered by this License; they are outside its scope.  The act of
running the Program is not restricted, and the output from the Program
is covered only if its contents constitute a work based on the
Program (independent of having been made by running the Program).
Whether that is true depends on what the Program does.

  1. You may copy and distribute verbatim copies of the Program's
source code as you receive it, in any medium, provided that you
conspicuously and appropriately publish on each copy an appropriate
copyright notice and disclaimer of warranty; keep intact all the
notices that refer to this License and to the absence of any warranty;
and give any other recipients of the Program a copy of this License
along with the Program.

You may charge a fee for the physical act of transferring a copy, and
you may at your option offer warranty protection in exchange for a fee.

  2. You may modify your copy or copies of the Program or any portion
of it, thus forming a work based on the Program, and copy and
distribute such modifications or work under the terms of Section 1
above, provided that you also meet all of these conditions:

    a) You must cause the modified files to carry prominent notices
    stating that you changed the files and the date of any change.

    b) You must cause any work that you distribute or publish, that in
    whole or in part contains or is derived from the Program or any
    part thereof, to be licensed as a whole at no charge to all third
    parties under the terms of this License.

    c) If the modified program normally reads commands interactively
    when run, you must cause it, when started running for such
    interactive use in the most ordinary way, to print or display an
    announcement including an appropriate copyright notice and a
    notice that there is no warranty (or else, saying that you provide
    a warranty) and that users may redistribute the program under
    these conditions, and telling the user how to view a copy of this
    License.  (Exception: if the Program itself is interactive but
    does not normally print such an announcement, your work based on
    the Program is not required to print an announcement.)

These requirements apply to the modified work as a whole.  If
identifiable sections of that work are not derived from the Program,
and can be reasonably considered independent and separate works in
themselves, then this License, and its terms, do not apply to those
sections when you distribute them as separate works.  But when you
distribute the same sections as part of a whole which is a work based
on the Program, the distribution of the whole must be on the terms of
this License, whose permissions for other licensees extend to the
entire whole, and thus to each and every part regardless of who wrote it.

Thus, it is not the intent of this section to claim rights or contest
your rights to work written entirely by you; rather, the intent is to
exercise the right to control the distribution of derivative or
collective works based on the Program.

In addition, mere aggregation of another work not based on the Program
with the Program (or with a work based on the Program) on a volume of
a storage or distribution medium does not bring the other work under
the scope of this License.

  3. You may copy and distribute the Program (or a work based on it,
under Section 2) in object code or executable form under the terms of
Sections 1 and 2 above provided that you also do one of the following:

    a) Accompany it with the complete corresponding machine-readable
    source code, which must be distributed under the terms of Sections
    1 and 2 above on a medium customarily used for software interchange; or,

    b) Accompany it with a written offer, valid for at least three
    years, to give any third party, for a charge no more than your
    cost of physically performing source distribution, a complete
    machine-readable copy of the corresponding source code, to be
    distributed under the terms of Sections 1 and 2 above on a medium
    customarily used for software interchange; or,

    c) Accompany it with the information you received as to the offer
    to distribute corresponding source code.  (This alternative is
    allowed only for noncommercial distribution and only if you
    received the program in object code or executable form with such
    an offer, in accord with Subsection b above.)

The source code for a work means the preferred form of the work for
making modifications to it.  For an executable work, complete source
code means all the source code for all modules it contains, plus any
associated interface definition files, plus the scripts used to
control compilation and installation of the executable.  However, as a
special exception, the source code distributed need not include
anything that is normally distributed (in either source or binary
form) with the major components (compiler, kernel, and so on) of the
operating system on which the executable runs, unless that component
itself accompanies the executable.

If distribution of executable or object code is made by offering
access to copy from a designated place, then offering equivalent
access to copy the source code from the same place counts as
distribution of the source code, even though third parties are not
compelled to copy the source along with the object code.

  4. You may not copy, modify, sublicense, or distribute the Program
except as expressly provided under this License.  Any attempt
otherwise to copy, modify, sublicense or distribute the Program is
void, and will automatically terminate your rights under this License.
However, parties who have received copies, or rights, from you under
this License will not have their licenses terminated so long as such
parties remain in full compliance.

  5. You are not required to accept this License, since you have not
signed it.  However, nothing else grants you permission to modify or
distribute the Program or its derivative works.  These actions are
prohibited by law if you do not accept this License.  Therefore, by
modifying or distributing the Program (or any work based on the
Program), you indicate your acceptance of this License to do so, and
all its terms and conditions for copying, distributing or modifying
the Program or works based on it.

  6. Each time you redistribute the Program (or any work based on the
Program), the recipient automatically receives a license from the
original licensor to copy, distribute or modify the Program subject to
these terms and conditions.  You may not impose any further
restrictions on the recipients' exercise of the rights granted herein.
You are not responsible for enforcing compliance by third parties to
this License.

  7. If, as a consequence of a court judgment or allegation of patent
infringement or for any other reason (not limited to patent issues),
conditions are imposed on you (whether by court order, agreement or
otherwise) that contradict the conditions of this License, they do not
excuse you from the conditions of this License.  If you cannot
distribute so as to satisfy simultaneously your obligations under this
License and any other pertinent obligations, then as a consequence you
may not distribute the Program at all.  For example, if a patent
license would not permit royalty-free redistribution of the Program by
all those who receive copies directly or indirectly through you, then
the only way you could satisfy both it and this License would be to
refrain entirely from distribution of the Program.

If any portion of this section is held invalid or unenforceable under
any particular circumstance, the balance of the section is intended to
apply and the section as a whole is intended to apply in other
circumstances.

It is not the purpose of this section to induce you to infringe any
patents or other property right claims or to contest validity of any
such claims; this section has the sole purpose of protecting the
integrity of the free software distribution system, which is
implemented by public license practices.  Many people have made
generous contributions to the wide range of software distributed
through that system in reliance on consistent application of that
system; it is up to the author/donor to decide if he or she is willing
to distribute software through any other system and a licensee cannot
impose that choice.

This section is intended to make thoroughly clear what is believed to
be a consequence of the rest of this License.

  8. If the distribution and/or use of the Program is restricted in
certain countries either by patents or by copyrighted interfaces, the
original copyright holder who places the Program under this License
may add an explicit geographical distribution limitation excluding
those countries, so that distribution is permitted only in or among
countries not thus excluded.  In such case, this License incorporates
the limitation as if written in the body of this License.

  9. The Free Software Foundation may publish revised and/or new versions
of the General Public License from time to time.  Such new versions will
be similar in spirit to the present version, but may differ in detail to
address new problems or concerns.

Each version is given a distinguishing version number.  If the Program
specifies a version number of this License which applies to it and "any
later version", you have the option of following the terms and conditions
either of that version or of any later version published by the Free
Software Foundation.  If the Program does not specify a version number of
this License, you may choose any version ever published by the Free Software
Foundation.

  10. If you wish to incorporate parts of the Program into other free
programs whose distribution conditions are different, write to the author
to ask for permission.  For software which is copyrighted by the Free
Software Foundation, write to the Free Software Foundation; we sometimes
make exceptions for this.  Our decision will be guided by the two goals
of preserving the free status of all derivatives of our free software and
of promoting the sharing and reuse of software generally.

			    NO WARRANTY

  11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
REPAIR OR CORRECTION.

  12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

		     END OF TERMS AND CONDITIONS
******************************************************************************/
